/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jclouds.iam.xml;

import static org.jclouds.util.SaxUtils.currentOrNull;
import static org.jclouds.util.SaxUtils.equalsOrSuffix;

import org.jclouds.collect.IterableWithMarker;
import org.jclouds.collect.IterableWithMarkers;
import org.jclouds.http.functions.ParseSax;
import org.xml.sax.Attributes;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableList.Builder;

/**
 * @see <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_ListGroupPolicies.html" />
 */
public class ListPoliciesResultHandler extends
      ParseSax.HandlerForGeneratedRequestWithResult<IterableWithMarker<String>> {

   private StringBuilder currentText = new StringBuilder();
   private Builder<String> names = ImmutableList.<String> builder();
   private boolean inPolicyNames;
   private String afterMarker;

   @Override
   public IterableWithMarker<String> getResult() {
      try {
         return IterableWithMarkers.from(names.build(), afterMarker);
      } finally {
         names = ImmutableList.<String> builder();
      }
   }

   @Override
   public void startElement(String url, String name, String qName, Attributes attributes) {
      if (equalsOrSuffix(qName, "PolicyNames")) {
         inPolicyNames = true;
      }
   }

   @Override
   public void endElement(String uri, String name, String qName) {
      if (inPolicyNames && qName.equals("PolicyNames")) {
         inPolicyNames = false;
      } else if (qName.equals("member")) {
         names.add(currentOrNull(currentText));
      } else if (qName.equals("Marker")) {
         afterMarker = currentOrNull(currentText);
      }
      currentText.setLength(0);
   }

   @Override
   public void characters(char[] ch, int start, int length) {
      currentText.append(ch, start, length);
   }
}
